/*
 * Copyright (c) 2012, Silvio Vallorani <silvio.vallorani@studio.unibo.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/*
 * Simple Demo Application

 *
 * This simple application will "shake" all the haptors present,
 * using constant pattern with different magnitude and then stop
 * them.
 *
 * Silvio Vallorani
 *
 * version 0.7
 * 2012
 *
 */

#include "stm32f10x.h"

#include "hapticLib.h"

enum demoHaptors {
	One   = 0,
	Two   = 1,
	Three = 2,
	Four  = 3,

	NumDemoHaptors = 4
};

int main(void)
{
	/* 
	 * Here the System Clock must be already configured.
   	 * Make sure the startup code (vector.c is the default) calls SystemInit()
   	 * before branching here to main().
   	 */

	// Structure to hold info about the haptors used in this demo
	haptor_desc *demoHaptors;
	
	// Structure to provide the only user parameter "constant"
	user_param myParams;

	uint16_t delay_between_patterns = 1000;
	uint8_t i;

	// Initialize the Haptic System (setting the PWM global period in Hz)
	demoHaptors = hl_configure(24000000,10, NumDemoHaptors);

    // START SHAKING ALL THE HAPTORS WITH DIFFERENT MAGNITUDO

	myParams.constant.constant = 15000;
	// Shake haptor 'One' with 'Constant' Pattern @ 15000.
	if( hl_startPattern( hl_addHaptor(&demoHaptors[One], hl_initPattern(Constant,&myParams) ) ) )
	{
#ifdef HL_DEBUG
		send_string("HapticConstant: Problem sending pattern!\r\n\0");
#endif
	}
	Delay(delay_between_patterns);

	myParams.constant.constant = 30000;
	// Shake haptor id='Two' with 'Constant' Pattern @ 30000.
	if( hl_startPattern( hl_addHaptor(&demoHaptors[Two], hl_initPattern(Constant,&myParams) ) ) )
	{
#ifdef HL_DEBUG
		send_string("HapticConstant: Problem sending pattern!\r\n\0");
#endif
	}
	Delay(delay_between_patterns);

	myParams.constant.constant = 45000;
	// Shake haptor id='Three' with 'Constant' Pattern @ 45000.
	if( hl_startPattern( hl_addHaptor(&demoHaptors[Three], hl_initPattern(Constant,&myParams) ) ) )
	{
#ifdef HL_DEBUG
		send_string("HapticConstant: Problem sending pattern!\r\n\0");
#endif
	}
	Delay(delay_between_patterns);

	myParams.constant.constant = 60000;
	// Shake haptor id='Four' with 'Constant' Pattern @ 60000.
	if( hl_startPattern( hl_addHaptor(&demoHaptors[Four], hl_initPattern(Constant,&myParams) ) ) )
	{
#ifdef HL_DEBUG
		send_string("HapticConstant: Problem sending pattern!\r\n\0");
#endif
	}
	Delay(delay_between_patterns);

    // OFF ALL THE HAPTORS
	for( i=0 ; i<NumDemoHaptors ; i++)
	{
		if(hl_stopPattern(demoHaptors[i].activePattern) )
		{
#ifdef HL_DEBUG
			send_string("HapticConstant: Problem stopping pattern!\r\n\0");
#endif
		}
	}

#ifdef HL_DEBUG
	send_string("HapticConstant: Going to sleep...goodbye!\r\n\0");
#endif

	while(1) { __WFE(); }
}
