/*
 * Copyright (c) 2012, Silvio Vallorani <silvio.vallorani@studio.unibo.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/*
 * Impact Patterns Demo
 *
 * This sample application will send various Impact Haptic Feedback Patterns
 * to a single haptor that reproduce impact to various materials at different
 * velocity.
 *
 * Leonardo Guardati
 * Silvio Vallorani
 * version 0.7
 * 2012
 *
 */

#include "stm32f10x.h"
#include "hapticLib.h"

int main(void)
{
	/*
	 * Here the System Clock must be already configured.
   	 * Make sure the startup code (vector.c is the default) calls SystemInit()
   	 * before branching here to main().
   	 */

	// Structure to hold info about a single haptor.
	haptor_desc *myHaptor;

	pattern_desc *myTestPattern;

	pattern_desc *myImpactPattern;
	user_param    myImpactParams;

	// Initialize the Haptic System (setting the PWM global period in KHz)
	// in a single haptor configuration.
	myHaptor = hl_configure(24000000,10,1);

#ifdef HL_DEBUG
	send_string("\r\n\r\n\tHello Impact Patterns!!!\r\n\0");
	send_string("\t\tShake haptor with 'Test' Pattern...\r\n\r\n\0");
#endif

	myTestPattern = hl_initPattern(Test, NULL);
	hl_addHaptor(myHaptor,myTestPattern);
	hl_startPattern(myTestPattern);

	Delay(5000);

	myImpactParams.impact.velocity = Fast;

	Delay(5000);

#ifdef HL_DEBUG
	send_string("\tShake haptor with Impact Pattern using various materials:\r\n\0");
#endif

#ifdef HL_DEBUG
	send_string("\t\tRUBBER\r\n\0");
#endif

	myImpactParams.impact.material = Rubber;
	myImpactPattern = hl_initPattern(Impact, &myImpactParams);
	hl_addHaptor(myHaptor,myImpactPattern);
	hl_startPattern(myImpactPattern);

	Delay(5000);

#ifdef HL_DEBUG
	send_string("\t\tWOOD\r\n\0");
#endif

	myImpactParams.impact.material = Wood;
	myImpactPattern = hl_initPattern(Impact, &myImpactParams);
	hl_addHaptor(myHaptor,myImpactPattern);
	hl_startPattern(myImpactPattern);

	Delay(5000);

#ifdef HL_DEBUG
	send_string("\t\tALUMINUM\r\n\0");
#endif

	myImpactParams.impact.material = Aluminum;
	// This does the same thing but the code is more dense
	hl_startPattern(
			hl_addHaptor(myHaptor,
					hl_initPattern(Impact, &myImpactParams) ) );

	Delay(5000);

#ifdef HL_DEBUG
	send_string("\r\nGoodbye....going to sleep...\r\n\0");
#endif

	while(1) { __WFE(); }
}
