/*
 *
 *  Created on: 19/apr/12
 *      Author: bojan
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __LIS3LV02DL_H
#define __LIS3LV02DL_H

/* Includes ------------------------------------------------------------------*/
#include "HAL_LIS3LV02DL.h"


/**
  * @defgroup Accelerometer_Configuration_Defines
  *@{
  */

/**
* \brief accelerometer sensitivity with 2 g full scale [mg/LSB]
*/
//#define LSM_Acc_Sensitivity_2g     1  /*!< accelerometer sensitivity with 2 g full scale [mg/LSB] */
//#define LSM_Acc_Sensitivity_4g     2  /*!< accelerometer sensitivity with 4 g full scale [mg/LSB] */
//#define LSM_Acc_Sensitivity_8g     3.9 /*!< accelerometer sensitivity with 8 g full scale [mg/LSB] */

#define LIS_Acc_Sensitivity_2g		1
#define LIS_Acc_Sensitivity_6g		2.9

#define LIS_Acc_Coeff_2g 			0.0005995

#define LIS_Acc_PowerOn				((u8)0xC0)
#define LIS_Acc_PowerOff			((u8)0x20)

#define LIS_Acc_ODR_40                          ((u8)0x00)
#define LIS_Acc_ODR_160                         ((u8)0x10)
#define LIS_Acc_ODR_640                         ((u8)0x20)
#define LIS_Acc_ODR_2560                        ((u8)0x40)

#define LIS_Acc_XEN                             ((u8)0x01)
#define LIS_Acc_YEN                             ((u8)0x02)
#define LIS_Acc_ZEN                             ((u8)0x04)
#define LIS_Acc_XYZEN                           ((u8)0x07)

#define LIS_Acc_FS_2                            ((u8)0x00)
#define LIS_Acc_FS_6                            ((u8)0x80)

#define LIS_Acc_BDU_Continuos                   ((u8)0x00)
#define LIS_Acc_BDU_Single                      ((u8)0x40)

#define LIS_Acc_Little_Endian                   ((u8)0x00)
#define LIS_Acc_Big_Endian                      ((u8)0x20)

#define LIS_Acc_Int_Enable		                ((u8)0x00)
#define LIS_Acc_Int_Disable                     ((u8)0x08)

#define LIS_Acc_DRDY_Enable		                ((u8)0x00)
#define LIS_Acc_DRDY_Disable                    ((u8)0x04)

#define LIS_Acc_SPI_3wire		                ((u8)0x02)
#define LIS_Acc_SPI_4wire	                    ((u8)0x00)

#define LIS_Acc_Data_Alignment_12				((u8)0x00)
#define LIS_Acc_Data_Alignment_16				((u8)0x01)


#define LIS_Acc_CLK_INT							((u8)0x00)
#define LIS_Acc_CLK_EXT							((u8)0x80)

#define LIS_Acc_HPDD_Enable						((u8)0x40)
#define LIS_Acc_HPDD_Disable					((u8)0x00)

#define LIS_Acc_HPFF_Enable						((u8)0x20)
#define LIS_Acc_HPFF_Disable					((u8)0x00)

#define LIS_Acc_Filter_Enable                   ((u8)0x10)
#define LIS_Acc_Filter_Disable                  ((u8)0x00)

#define LIS_Acc_Filter_HPc512                     ((u8)0x00)
#define LIS_Acc_Filter_HPc1024                    ((u8)0x01)
#define LIS_Acc_Filter_HPc2048                    ((u8)0x02)
#define LIS_Acc_Filter_HPc4096                    ((u8)0x03)


#define LIS_Acc_OffsetX_Default		0
#define LIS_Acc_OffsetY_Default		0
#define LIS_Acc_OffsetZ_Default		0

#define LIS_Acc_Gain00_Default		1
#define LIS_Acc_Gain01_Default		0
#define LIS_Acc_Gain02_Default		0
#define LIS_Acc_Gain10_Default		0
#define LIS_Acc_Gain11_Default		1
#define LIS_Acc_Gain12_Default		0
#define LIS_Acc_Gain20_Default		0
#define LIS_Acc_Gain21_Default		0
#define LIS_Acc_Gain22_Default		1

/**
  * @}
  */ /* end of group Accelerometer_Configuration_Defines */

/** @defgroup Accelerometer_Register_Mapping
  * @{
  */


/**
*  \brief Accelerometer I2C Slave Address
*/
#define LIS_A_I2C_ADDRESS         0x3A

/**
*  \brief Accelerometer Who am I register
*/

#define LIS_A_WHO_AM_I			0x0F

/**
*  \brief Accelerometer Control Register 1
*  \code
*   Read Write
*   Default value: 0x07
*   7:6 PD1-PD0: Power down control.
*   		   PD1 |  PD0 |   Power Mode
          	  ----------------------------
*               0  |  0   |  Power Down	(default)
*               0  |  1   |  Power ON
*               1  |  0   |  Power ON
*               1  |  1   |  Power ON
*
*   5:4 DF1-DF0: Decimation Factor control, sets the data rate
*   		   DF1 |  DF0 |   Data Rate
          	  ----------------------------
*               0  |  0   |  40 Hz (default)
*               0  |  1   |  160 Hz
*               1  |  0   |  640
*               1  |  1   |  2560
*
*   3 ST: Self Test
*   2 Zen: Z axis enable. 0 - Z axis disabled  1- Z axis enabled
*   1 Yen: Y axis enable. 0 - Y axis disabled  1- Y axis enabled
*   0 Xen: X axis enable. 0 - X axis disabled  1- X axis enabled
*   \endcode
*/
#define LIS_A_CTRL_REG1_ADDR     0x20

/**
*   \brief Accelerometer Control Register 2
*   \code
*   Read Write
*   Default value: 0x00
*
*   7 FS full scale selection: 0=2g, 1=6g
*   6 BDU Block data update. 0 -continuos update; 1- output registers not updated between MSB and LSB reading
*   5 BLE  Big/little endian data selection. 0 - data LSB @ lower address; 1 - data MSB @ lower address
*   4 BOOT: Reboot memory content 0 - normal mode  1 - reboot memory content
*   3 IEN: Interrupt enable 0: data ready on RDY pad; 1: interrupt events on RDY pad
*   2 DRDY Enable Data-Ready generation
*   1 SIM: SPI Serial Interface Mode selection (0: 4-wire interface; 1: 3-wire interface)
*   0 DAS: Data alignment selection (0: 12 bit right justified; 1: 16 bit left justified)
*   \endcode
*   */
#define LIS_A_CTRL_REG2_ADDR     0x21

/**
*  \brief Accelerometer Control Register 3
*  \code
*   Read Write
*  Default value: 0x00
*
*
*  7 ECK: External Clock. Default value: 0 (0: clock from internal oscillator; 1: clock from external pad)
*  6 HPDD: High Pass filter enabled for Direction Detection. Default value: 0 (0: filter bypassed; 1: filter enabled)
*  5 HPFF: High Pass filter enabled for Free-Fall and Wake-Up. Default value: 0 (0: filter bypassed; 1: filter enabled)
*  4 FDS: Filtered Data Selection. Default value: 0 (0: internal filter bypassed; 1: data from internal filter)
*  3-2 Reserved Bits
*  1-0 CFS1-CFS0: High-pass filter Cut-off Frequency Selection. Default value: 00
*
*  			f_cutoff = (0.318 / Hpc)*(ODRx / 2)
*  			  CFS1 |  CFS0 |   Hpc
*          	  ----------------------------
*               0  |  0    |   512
*               0  |  1    |  1024
*               1  |  0    |  2048
*               1  |  1    |  4096
*
*   \endcode
*/
#define LIS_A_CTRL_REG3_ADDR     0x22


/**
*  \brief Accelerometer HP Filter Reset Register
*  \code
*  Dummy register. Reading at this address zeroes instantaneously the content of the internal
*  high pass-filter. If the high pass filter is enabled all three axes are instantaneously set to 0g.
*  This allows to overcome the settling time of the high pass filter.
*  Read
*  Default value: 0x00
* \endcode
*/
#define LIS_A_HP_FILTER_RESET_REG_ADDR     0x23


/**
*  \brief Accelerometer Status Register

*/

#define LIS_A_STATUS_REG_ADDR     0x27


/**
*  \brief Accelerometer X axis Output Data LSB Register
*  \code
*  Read
*  Default value: ( The value is expressed as 16bit twos complement)
*  7:0 XOUT7-XOUT0: ACC Data LSB (if in Little Endian Mode --> BLE bit in CTRL_REG4 is 0)
*                   ACC Data MSB (if in Big Endian Mode --> BLE bit in CTRL_REG4 is 1)
* \endcode
*/
#define LIS_A_OUT_X_L_ADDR     0x28

/**
*  \brief  Acceleration X-axis Output Data MSB Register
*  \code
*  Default value: ( The value is expressed as 16bit twos complement)
*  7:0 XOUT15-XOUT8: ACC Data MSB (if in Little Endian Mode --> BLE bit in CTRL_REG1 is 0)
*                    ACC Data LSB (if in Big Endian Mode --> BLE bit in CTRL_REG1 is 1)
* \endcode
*/
#define LIS_A_OUT_X_H_ADDR     0x29

/**
*  \brief Acceleration Y-axis Output Data LSB Register
*  \code
*  Read
*  Default value: ( The value is expressed as 16bit twos complement)
*  7:0 YOUT7-YOUT0: ACC Data LSB (if in Little Endian Mode --> BLE bit in CTRL_REG4 is 0)
*                   ACC Data MSB (if in Big Endian Mode --> BLE bit in CTRL_REG4 is 1)
* \endcode
*/
#define LIS_A_OUT_Y_L_ADDR     0x2A

/**
*  \brief  Acceleration Y-axis Output Data MSB Register
*  \code
*   Read register
*   Default value:  ( The value is expressed as 16bit twos complement)
*   7:0 YOUT15-YOUT8: ACC Data MSB (if in Little Endian Mode --> BLE bit in CTRL_REG1 is 0)
*                   ACC Data LSB (if in Big Endian Mode --> BLE bit in CTRL_REG1 is 1)
* \endcode
*/
#define LIS_A_OUT_Y_H_ADDR     0x2B


/**
*  \brief  Acceleration Z-axis Output Data LSB Register
*  \code
*  Read
*  Default value: ( The value is expressed as 16bit twos complement)
*  7:0 YOUT7-YOUT0: ACC Data LSB (if in Little Endian Mode --> BLE bit in CTRL_REG4 is 0)
*                   ACC Data MSB (if in Big Endian Mode --> BLE bit in CTRL_REG4 is 1)
* \endcode
*/
#define LIS_A_OUT_Z_L_ADDR     0x2C

/**
*  \brief  Acceleration Z-axis Output Data MSB Register
*  \code
*  Default value: ( The value is expressed as 16bit twos complement)
*  7:0 YOUT15-YOUT8: ACC Data MSB (if in Little Endian Mode --> BLE bit in CTRL_REG1 is 0)
*                   ACC Data LSB (if in Big Endian Mode --> BLE bit in CTRL_REG1 is 1)
*   \endcode
*/
#define LIS_A_OUT_Z_H_ADDR     0x2D

/**
*  \brief Accelerometer Configuration Register for Free fall and inertial wake up configuration.
*
*/
#define LIS_A_FF_WU_CFG_REG_ADDR 0x30


/**
*  \brief Accelerometer Configuration Register for Free fall and inertial wake up interrupt source.
*
*/
#define LIS_A_FF_WU_SRC_REG_ADDR 0x31

/**
*  \brief Dummy - reading at this address allows the FF_WU_SRC register refresh
*
*/
#define LIS_A_FF_WU_ACK_REG_ADDR 0x32

/**
*  \brief Free-fall / Inertial Wake Up Acceleration Threshold LSB
*
*/
#define LIS_A_FF_WU_THS_L_REG_ADDR 0x34

/**
*  \brief Free-fall / Inertial Wake Up Acceleration Threshold MSB
*
*/
#define LIS_A_FF_WU_THS_H_REG_ADDR 0x35

/**
*  \brief Free-fall / Inertial Wake Up Event minimum duration
*
*  Duration(s) = FF_WU_DURATION(Dec)/ODR
*
*/
#define LIS_A_FF_WU_DURATION_REG_ADDR 0x36


/**
*  \brief Direction detecor configuration register
*
*/
#define LIS_A_DD_CFG_REG_ADDR 0x38

/**
*  \brief Direction detecor source register
*
*/
#define LIS_A_DD_SRC_REG_ADDR 0x39

/**
*  \brief Dummy - reading at this address allows the DD_SRC register refresh
*
*/
#define LIS_A_DD_ACK_REG_ADDR 0x3A

/**
*  \brief Direction detection Internal Threshold LSB
*
*/
#define LIS_A_DD_THSI_L_REG_ADDR 0x3C

/**
*  \brief Direction detection Internal Threshold MSB
*
*/
#define LIS_A_DD_THSI_H_REG_ADDR 0x3D

/**
*  \brief Direction detection External Threshold LSB
*
*/
#define LIS_A_DD_THSE_L_REG_ADDR 0x3E

/**
*  \brief Direction detection External Threshold MSB
*
*/
#define LIS_A_DD_THSE_H_REG_ADDR 0x3F

/**
  * @}
  */ /* end of group Accelerometer_Register_Mapping */


/**
* @brief Accelerometer Init structure definition
*/

typedef struct
{

  u8 Power_Mode;  /*!<  Low power mode selection (see table 19 datasheet) */
  u8 ODR; /*!< Output Data Rate */
  u8 Axes_Enable;    /*!< Axes Enable */
  u8 FS;     /*!< Full Scale */
  u8 Data_Update;  /*!< Data Update mode : Continuos update or data don`t change until MSB and LSB nex reading */
  u8 Endianess;   /*!< Endianess */
  u8 Interrupt;
  u8 Data_Ready;
  u8 Serial_Mode;
  u8 Data_Alignment;
}LIS_Acc_ConfigTypeDef;



/**
  * @brief Accelerometer  Filter Init structure definition
  */


typedef struct
{
  u8 CLK_selsction;
  u8 HPF_Enable; /*!< HPF enable*/
  u8 HPFF;
  u8 HPDD;
  //u8 HPF_Mode;  /*!<HPF MODE: Normal mode or Reference signal for filtering*/
  //u8 HPF_Reference;     /*!< Reference value for filtering*/
  u8 HPF_Frequency;     /*!< HPF_frequency ft=ODR/6*HPc  HPc=8,16,32,64*/
}LIS_Acc_Filter_ConfigTypeDef;


/**
 * @}
 */ /* end of group Accelerometer */



void LIS3LV02DL_I2C_ByteWrite(u8 slAddr, u8* pBuffer, u8 WriteAddr);
void LIS3LV02DL_I2C_BufferRead(u8 slAddr, u8* pBuffer, u8 ReadAddr, u16 NumByteToRead);
void LIS3LV02DL_I2C_Init(void);

void LIS3LV02DL_Acc_Init (void);
void LIS3LV02DL_Acc_Config (LIS_Acc_ConfigTypeDef *LIS_Acc_Config_Struct);
void LIS3LV02DL_Acc_Filter_Config (LIS_Acc_Filter_ConfigTypeDef *LIS_Acc_Filter_Config_Struct);
//void LIS3LV02DL_Acc_Lowpower_Cmd (u8 LowPowerMode);
void LIS3LV02DL_Acc_FullScale_Cmd (u8 FS_value);
void LIS3LV02DL_Acc_DataRate_Cmd (u8 DataRateValue);
void LIS3LV02DL_Acc_Reboot_Cmd (void);
//void LIS3LV02DL_Acc_Read_OutReg (u8* out);
void LIS3LV02DL_Acc_Read_Buffer(u8* out);
void LIS3LV02DL_Acc_Read_RawData (s16* out, char dir);
void LIS3LV02DL_Acc_Read_Data(float* out);
//void LIS3LV02DL_Acc_Calibration(s16* raw_data, float* out);

#endif /* __LIS3LV02DL_H */
