/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * Simple Demo Application
 *
 * This simple application will "blink" one haptor using
 * a simple Test pattern that doesn't need any user
 * parameters.
 *
 * It shows how simple is to interact with HapticLib.
 *
 * Please note that this is the basic way of using
 * HapticLib. See other Demos to know all the other features
 * available, like multi-haptor support, debugging, error
 * signaling.
 *
 * Guardati Leonardo
 *
 * version 0.7
 * 2012
 *
 */

#include "hapticLib.h"

enum demoHaptors {
	Head          = 0,
	Lhand         = 1,
	Rhand         = 2,
	Back          = 3,

	NumDemoHaptor = 4
};

int main(void)
{
	/* 
	 * Here the System Clock must be already configured.
   	 * Make sure the startup code (vector.c is the default) calls SystemInit()
   	 * before branching here to main().
   	 */

	// Pointer to the array containing the setup haptors.
	haptor_desc  *myHaptors;

	// Pointer to the Test pattern descriptor
	pattern_desc *myPattern;
	
	// Initialize the Haptic System and get the haptors array.
	myHaptors = hl_configure(24000000, 10, NumDemoHaptor);

	// We are ready to send patterns.

	// Use a Test Pattern
	myPattern = hl_initPattern(Test, NULL);

	// We need to tell the Test pattern which haptor to use
	hl_addHaptor( &myHaptors[Head], myPattern);

	// Start Shaking
	hl_startPattern(myPattern);

	// Let it shake for a while...
	Delay(1000);

	// ...or force its stop if we need the haptor for something else.
	hl_stopPattern(myPattern);

#ifdef HL_DEBUG
		send_string("\r\nHapticWorld: Going to sleep...goodbye!\r\n\0");
#endif

	// go to sleep
	while(1) { __WFE(); }
}
