/*
 * Copyright (c) 2012, Silvio Vallorani <silvio.vallorani@studio.unibo.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file impact.h
 *
 * @brief **Impact Pattern** generator function header.
 *
 * Here impactPatternParameters type is defined.
 *
 * @author Silvio Vallorani
 * @version v0.7
 * @date 2012
 *
 */

/**
 * @brief User Parameter Values for **Material** user parameter.
 *
 * Enumeration of material types that can be passed as
 * parameter to Impact Pattern Generator in a user friendly
 * way.
 *
 */
typedef enum ImpactMaterial {
	Rubber   = 1,
	Wood     = 2,
	Aluminum = 3
} ImpactMaterial;

/**
 * @brief User Parameter Values for **Velocity** user parameter.
 *
 * Enumeration of velocities that can be passed as
 * parameter to Impact Pattern Generator in a user friendly
 * way.
 *
 */
typedef enum ImpactVelocity {
	Slow   = 1,
	Normal = 2,
	Fast   = 3
} ImpactVelocity;

 /**
  * @brief Struct for user provided parameters to hl_startPattern().
  *
  * Please refer to impact pattern generator
  * [optional parameters](@ref impact_opt_par) to know the
  * possible values.
  *
  */
typedef struct {
	ImpactVelocity velocity; /**< @brief Impact `velocity`
	 	 	 	 	 	 	 	  parameter. */
	ImpactMaterial material; /**< @brief Impact `material`
								  parameter. */
} impactUserParameters;

/**
 * @brief Impact pattern **status** parameters type definition.
 *
 * Only one parameter is defined to help the **continuator**
 * keep track of the pattern generation progress.
 *
 */
typedef struct {
	uint8_t progress;
} impactStatusParameters;
