/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file test.h
 *
 * @brief **Test Pattern** generator function header
 *
 * No Pattern specific **constant definitions** are needed
 * by this pattern.
 *
 * The Pattern specific **status parameters** typedef
 * is defined here.
 *
 * No Pattern specific **user parameters** typedef
 * is needed.
 *
 *
 * @authors Leonardo Guardati
 * @authors Silvio Vallorani
 * @version v0.7
 * @date 2012
 *
 */

/**
 * @brief Test Pattern specific **status** parameters typedef.
 *
 * Status parameter used by the continuator.
 *
 * @see Refer to generic.c for a simple working example.
 *
 */
typedef struct {
	uint8_t  flag; /**< @brief **status parameter** is used
						to implement the saw-tooth double ramp.*/
	uint16_t duty; /**< @brief **status parameter** is used to
						hold the new duty-cycle for the PWM
						and to keep track of the pattern
						generation progress. */
} testStatusParameters;
