/*
 * Copyright (c) 2012, Leonardo Guardati <leonardo@guardati.it>
 *
 * Permission  to  use,  copy,  modify,  and/or distribute this
 * software  for  any  purpose  with  or  without fee is hereby
 * granted,  provided  that the above copyright notice and this
 * permission notice appear in all copies.
 *
 * THE  SOFTWARE  IS  PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL  WARRANTIES  WITH  REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND FITNESS. IN NO
 * EVENT  SHALL  THE  AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT,   OR   CONSEQUENTIAL   DAMAGES   OR   ANY  DAMAGES
 * WHATSOEVER  RESULTING  FROM  LOSS  OF  USE, DATA OR PROFITS,
 * WHETHER  IN  AN  ACTION  OF  CONTRACT,  NEGLIGENCE  OR OTHER
 * TORTIOUS  ACTION,  ARISING  OUT OF OR IN CONNECTION WITH THE
 * USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
/**
 * @file    hl_STM32VLDISCOVERY.h
 *
 * @brief   **Platform Specific Module** STM32VLDISCOVERY header.
 *
 * This file is part of the **Platform Specific
 * Module** of HapticLib.
 *
 * HapticLib support for the STM32VL-DISCOVERY platform  is
 * documented on the [Architecture page](@ref stm32vldiscovery),
 * please refer to it for additional informations.
 *
 *
 * @author Leonardo Guardati
 * @version 0.7
 * @date 2012
 *
 */

/**
 * @brief Maximum number of Haptor supported in this Platform.
 *
 * At the moment **HapticLib STM32VLDISCOVERY Platform Specific
 * Module** supports only 4 Haptic devices.
 *
 * For hardware connections and details, please refer to
 * HapticLib documentation on the
 * [Architecture page](@ref stm32vldiscovery).
 *
 * @note following versions will increase the number of maximum
 *       haptors supported.
 *
 */
#define MAX_HAPTORS 4

/**
 * @brief Use the STM StdPeriph Library.
 *
 * This is the only dependency needed by HapticLib.
 * To successfully compile and use the library code,
 * you need to unpack the StdPeriph on the root of
 * HapticLib.
 *
 * @see Please refer to the [User Guide](@ref user_guide)
 *      for details on user environment setup.
 *
 */
#ifndef USE_STDPERIPH_DRIVER
#define USE_STDPERIPH_DRIVER
#endif

/**
 * @brief MCU device class needed by STM's StdPeriph.
 *
 * STM32 MCU device class define.
 * (MD: Medium Density, VL: Value Line)
 *
 * This symbol is required by StdPeriph, and this is
 * the MCU present on the STM32VLDISCOVERY board.
 *
 */
#ifndef STM32F10X_MD_VL
#define STM32F10X_MD_VL
#endif

#include "stm32f10x.h"

void    RCC_Configuration(void);
void    GPIO_Configuration(void);
void    TIM_Configuration(uint32_t);
uint8_t TIM_Channel_Enable(uint8_t);
uint8_t TIM_Channel_DutyChanger(uint16_t, uint8_t);
void    SysTick_Handler(void);
void    Delay(__IO uint32_t);
#ifdef HL_DEBUG
void    USART_Configuration(void);
uint8_t send_char(uint8_t ch);
#endif
